package Sys::CPUtime;

use 5.005;
use strict;

require Exporter;
require DynaLoader;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
@ISA = qw(Exporter DynaLoader);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Sys::CPUtime ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
%EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

@EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

@EXPORT = qw(
	
);

$VERSION = '0.10';

bootstrap Sys::CPUtime $VERSION;
#bootstrap CPUtime $VERSION;

1;

################################
# example of Sys::CPUtime usage:
# sub fork_before (time)
# if we run on a system with enabled CPU time limit 
# we can fork just before time limit will be exceeded.
# it allows us to continue the same operations but with new pid.
#
# It is template only, so write your own version,
# especially if you use DBI. In last case it is
# much safer to close your database connections 
# and reconnect after fork(). (bugs in some DBD's?)
################################
sub fork_before ($) {
    my $margin_time=shift;
    if (Sys::CPUtime::remained_time()<=$margin_time) {
	# DBI stuff
	exit (0) if fork();
	# DBI stuff
	return $$;
    } else {
	return 0 ;
    }
}

1;
__END__
=head1 NAME

Sys::CPUtime - Perl extension for SIGXCPU workaround

=head1 SYNOPSIS

  use Sys::CPUtime;

example of Sys::CPUtime usage is built-in sub

  Sys::CPUtime::fork_before(margin time in seconds) --

    checks time remaining before the process will get the SIGXCPU signal
    and forks if it is nesessary to survive (namely, when the remaining 
    time is less then the specified margin).

 It is template only, so write your own version,
 especially if you use DBI. In last case it is
 much safer to close your database connections 
 and reconnect after fork(). (bugs in some DBD's?)

 The useful functions are:

timelimit() returns time limit (getrlimit(RLIMIT_CPU, &limit)) or undef if none;
used_utime() returns used user time (getrusage (RUSAGE_SELF, &usage).ru_utime);
used_stime() returns used sys time (getrusage (RUSAGE_SELF, &usage).ru_stime);
process_time() returns used_utime()+used_stime();
remained_time() returns remained_time to get SIGXCPU 
(getrlimit - getrusage or INT_MAX if there is no rlimit)

=head1 DESCRIPTION

The Defence from Provider module provides perl API for unix
syscalls getrusage and getrlimit as long as SIGXCPU workaround.

if we run on a system with enabled CPU time limit 
we can fork just before time limit will be exceeded.
it allows us to continue the same operations but with new pid
to create persistent daemons on a hosting with CPU time limits.

It is tested on Linux and FreeBSD.

=head2 EXPORT

None by default.


=head1 AUTHOR

I. Yu. Vlasenko <lt>vlasenko@imath.kiev.ua<gt>
(c) 2002

=head1 LICENSE

Perl Artistics License

=head1 SEE ALSO

L<perl>.

=cut
